<?php

namespace Drupal\entityreference_modal_create\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\Validator\ConstraintViolationInterface;

/**
 * Plugin implementation of the 'entity_modal_widget' widget.
 *
 * @FieldWidget(
 *   id = "entity_modal_widget",
 *   module = "entityreference_modal_create",
 *   label = @Translation("Entity Reference Modal Create"),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class EntityModalWidget extends WidgetBase {


  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'match_operator' => 'CONTAINS',
        'match_limit' => 10,
        'size' => 60,
        'placeholder' => '',
        'allow_add_new' => 0,
        'add_new_button_title' => 'Add new',
        'selected_bundle' => '',
        'modal_form_mode' => 'default',
        'modal_width' => '80%',
        'modal_title' => 'Add new',
        'bypass_access_control' => 0,
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['match_operator'] = [
      '#type' => 'radios',
      '#title' => t('Autocomplete matching'),
      '#default_value' => $this->getSetting('match_operator'),
      '#options' => $this->getMatchOperatorOptions(),
      '#description' => t('Select the method used to collect autocomplete suggestions. Note that <em>Contains</em> can cause performance issues on sites with thousands of entities.'),
    ];
    $element['match_limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Number of results'),
      '#default_value' => $this->getSetting('match_limit'),
      '#min' => 0,
      '#description' => $this->t('The number of suggestions that will be listed. Use <em>0</em> to remove the limit.'),
    ];
    $element['size'] = [
      '#type' => 'number',
      '#title' => t('Size of textfield'),
      '#default_value' => $this->getSetting('size'),
      '#min' => 1,
      '#required' => TRUE,
    ];
    $element['placeholder'] = [
      '#type' => 'textfield',
      '#title' => t('Placeholder'),
      '#default_value' => $this->getSetting('placeholder'),
      '#description' => t('Text that will be shown inside the field until a value is entered. This hint is usually a sample value or a brief description of the expected format.'),
    ];

    $element['allow_add_new'] = [
      '#type' => 'checkbox',
      '#title' => t('Allow Add new'),
      '#default_value' => $this->getSetting('allow_add_new'),
    ];
    $element['bypass_access_control'] = [
      '#type' => 'checkbox',
      '#title' => t('Bypass Access control'),
      '#default_value' => $this->getSetting('bypass_access_control'),
    ];
    $element['add_new_button_title'] = [
      '#type' => 'textfield',
      '#title' => t("'Add' button title"),
      '#default_value' => $this->getSetting('add_new_button_title'),
      '#states' => [
        'invisible' => [
          ':input[data-drupal-selector="edit-fields-field-tags-settings-edit-form-settings-allow-add-new"]' => ['checked' => FALSE],
        ],
        'required' => [
          ':input[data-drupal-selector="edit-fields-field-tags-settings-edit-form-settings-allow-add-new"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $entityType = $this->getFieldSetting('target_type');
    $bundles = \Drupal::service('entity_type.bundle.info')->getBundleInfo($entityType);
    $modes = \Drupal::service('entity_display.repository')->getFormModeOptions($entityType);
    $options = [];
    foreach($bundles as $key => $bundle){
      $options[$key] = $bundle['label'];
    }
    $element['selected_bundle'] = [
      '#type' => 'radios',
      '#title' => t("Bundles allowed to be added"),
      '#options' => $options,
      '#default_value' => $this->getSetting('selected_bundle'),
      '#states' => [
        'invisible' => [
          ':input[data-drupal-selector="edit-fields-field-tags-settings-edit-form-settings-allow-add-new"]' => ['checked' => FALSE],
        ],
        'required' => [
          ':input[data-drupal-selector="edit-fields-field-tags-settings-edit-form-settings-allow-add-new"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $element['modal_form_mode'] = [
      '#type' => 'radios',
      '#title' => t("Form mode"),
      '#default_value' => $this->getSetting('modal_form_mode'),
      '#options' => $modes,
      '#states' => [
        'invisible' => [
          ':input[data-drupal-selector="edit-fields-field-tags-settings-edit-form-settings-allow-add-new"]' => ['checked' => FALSE],
        ],
        'required' => [
          ':input[data-drupal-selector="edit-fields-field-tags-settings-edit-form-settings-allow-add-new"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $element['modal_width'] = [
      '#type' => 'textfield',
      '#title' => t("Modal width"),
      '#default_value' => $this->getSetting('modal_width'),
      '#states' => [
        'invisible' => [
          ':input[data-drupal-selector="edit-fields-field-tags-settings-edit-form-settings-allow-add-new"]' => ['checked' => FALSE],
        ],
        'required' => [
          ':input[data-drupal-selector="edit-fields-field-tags-settings-edit-form-settings-allow-add-new"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $element['modal_title'] = [
      '#type' => 'textfield',
      '#title' => t("Modal title"),
      '#default_value' => $this->getSetting('modal_title'),
      '#states' => [
        'invisible' => [
          ':input[data-drupal-selector="edit-fields-field-tags-settings-edit-form-settings-allow-add-new"]' => ['checked' => FALSE],
        ],
        'required' => [
          ':input[data-drupal-selector="edit-fields-field-tags-settings-edit-form-settings-allow-add-new"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $operators = $this->getMatchOperatorOptions();
    $summary[] = t('Autocomplete matching: @match_operator', ['@match_operator' => $operators[$this->getSetting('match_operator')]]);
    $size = $this->getSetting('match_limit') ?: $this->t('unlimited');
    $summary[] = $this->t('Autocomplete suggestion list size: @size', ['@size' => $size]);
    $summary[] = t('Textfield size: @size', ['@size' => $this->getSetting('size')]);
    $placeholder = $this->getSetting('placeholder');
    if (!empty($placeholder)) {
      $summary[] = t('Placeholder: @placeholder', ['@placeholder' => $placeholder]);
    }
    else {
      $summary[] = t('No placeholder');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $referenced_entities = $items->referencedEntities();
    $form['#attached']['library'][] = 'entityreference_modal_create/referenced_entity_modal';
    $selection_settings = $this->getFieldSetting('handler_settings') + [
        'match_operator' => $this->getSetting('match_operator'),
        'match_limit' => $this->getSetting('match_limit'),
      ];
    $modal_selector = $items->getName(). "-" . $delta . "-" . rand(0,100000);
    \Drupal::service('page_cache_kill_switch')->trigger();
    $form['#cache']['max-age'] = 0;
    if($this->getSetting('bypass_access_control')){
      \Drupal::request()->getSession()->set($modal_selector,'bypassed');
    }
    $target = $element + [
      '#type' => 'entity_autocomplete',
      '#title_display' => 'invisible',
      '#target_type' => $this->getFieldSetting('target_type'),
      '#selection_handler' => $this->getFieldSetting('handler'),
      '#selection_settings' => $selection_settings,
      '#validate_reference' => FALSE,
      '#maxlength' => 1024,
      '#default_value' => isset($referenced_entities[$delta]) ? $referenced_entities[$delta] : NULL,
      '#size' => $this->getSetting('size'),
      '#placeholder' => $this->getSetting('placeholder'),
      '#attributes' => [
        '_delta' => $delta,
        'class' => ['referenced-entity-modal-target'],
        'style' => 'display: none;',
        '_modal_selector' => $modal_selector."-target-id",
      ],
      '#prefix' => '<div style="display: none !important;" class="hidden">',
      '#suffix' => '</div>',
    ];

    $mask = $element + [
      '#type' => 'entity_autocomplete',
      '#target_type' => $this->getFieldSetting('target_type'),
      '#selection_handler' => $this->getFieldSetting('handler'),
      '#selection_settings' => $selection_settings,
      '#maxlength' => 1024,
      '#default_value' => isset($referenced_entities[$delta]) ? $referenced_entities[$delta] : NULL,
      '#size' => $this->getSetting('size'),
      '#placeholder' => $this->getSetting('placeholder'),
      '#attributes' => [
        '_delta' => $delta,
        'class' => ['referenced-entity-modal-input'],
        '_modal_selector' => $modal_selector."-mask-input",
      ],
      '#validated' => TRUE,
      '#suffix' => '<span aria-hidden="true" class="field-modal-reset">×</span>',
    ];
    $output = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['referenced-entity-modal-container'],
      ],
      'target_id' => $target ,
      'mask_input' => $mask,
    ];
    if($this->getSetting('allow_add_new')) {
      $output['new_entity_button'] = [
        '#type' => 'link',
        '#weight' => 999,
        '#title' => $this->getSetting('add_new_button_title'),
        '#url' => Url::fromRoute('entityreference_modal_create.modal_entity_form', [
          'entityType' => $this->getFieldSetting('target_type'),
          'bundle' => $this->getSetting('selected_bundle'),
          'mode' => $this->getSetting('modal_form_mode'),
          'selector' => $modal_selector,
        ]),
        '#attributes' => [
          '_delta' => $delta,
          'class' => ['use-ajax', 'button btn'],
          'data-dialog-type' => 'modal',
          'data-dialog-options' => json_encode([
            'title' => $this->t($this->getSetting('modal_title')),
            'width' => $this->getSetting('modal_width'),
            'dialogClass'=> 'entity-modal-dialog',
            'classes' => [
              'ui-dialog-titlebar-close' => 'close-entity-modal',
            ],
          ]),
        ],
      ];
    }
    return $output;
  }

  /**
   * {@inheritdoc}
   */
  public function errorElement(array $element, ConstraintViolationInterface $error, array $form, FormStateInterface $form_state) {
    return $element['target_id'];
  }


  /**
   * Returns the value of a setting for the entity reference selection handler.
   *
   * @param string $setting_name
   *   The setting name.
   *
   * @return mixed
   *   The setting value.
   */
  protected function getSelectionHandlerSetting($setting_name) {
    $settings = $this->getFieldSetting('handler_settings');
    return isset($settings[$setting_name]) ? $settings[$setting_name] : NULL;
  }

  /**
   * Returns the options for the match operator.
   *
   * @return array
   *   List of options.
   */
  protected function getMatchOperatorOptions() {
    return [
      'STARTS_WITH' => t('Starts with'),
      'CONTAINS' => t('Contains'),
    ];
  }
}
